/*
   $Id: ParametersIndex.cpp 666 2011-11-15 10:16:53Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/
#include "ParametersIndex.h"
#include "Command.h"

CSimpleOpt ParametersInitializerIndex::_Init(int argc, char* argv[], const Command& cmd)
{
	CSimpleOpt::SOption glint_options[] = {
	    // ID       TEXT          TYPE
	    { OPT_COMPLEXITY,    (char*) "-C",   SO_REQ_SEP }, // "-C"
	    { OPT_LC_FILTERING,  (char*) "--no-lc-filtering", SO_NONE }, // "--no-lc-filtering"
	    { OPT_SILENT,  (char*)"--silent",   SO_NONE }, //
	    { OPT_VERBOSE,  (char*)"--verbose", SO_NONE }, //
	    { OPT_VERBOSE,  (char*)"-v",        SO_NONE }, //
	
	#ifdef DEBUGLOG
	    { OPT_LOG_LEVEL,  (char*)"-D",     SO_REQ_SEP }, // "-D"
	#endif
	
		// writing compressed data
		//{ OPT_COMPRESS,   (char*) "--compress",     SO_REQ_SEP },
		
	    { OPT_MAX_COUNT, (char*) "-n",     SO_REQ_SEP    }, // "-n"
	    { OPT_MASK,  (char*) "-M",     SO_REQ_SEP    },     // "-M"
	    { OPT_MASK,  (char*) "--mask", SO_REQ_SEP    },     // "---mask"
	    { OPT_BANK_NAME,  (char*)"-N",     SO_REQ_SEP   },  // "-w"
	    { OPT_NB_PASS,  (char*)"-p",     SO_REQ_SEP    },   // "-p"
	    
	    // To get some help
	    { OPT_HELP, (char*)"-?",     SO_NONE    }, // "-?"
	    { OPT_HELP, (char*)"-h",     SO_NONE    }, // "-?"
	    { OPT_HELP, (char*)"--help", SO_NONE    }, // "--help"
	    
	    SO_END_OF_OPTIONS                       // END
	};

	//p.__Help = help;
	p.__argc = argc;

	// declare our options parser, pass in the arguments from main
    // as well as our array of valid options.
    CSimpleOpt args(argc, argv, glint_options);
	while (args.Next()) {
		if (args.LastError() == SO_SUCCESS) 
		{
			if (args.OptionId() == OPT_HELP) 
			{
				cmd.Usage();
				exit(0);
			}
			else if (args.OptionId() == OPT_COMPLEXITY )
			{
				p.__complexity = atoi(args.OptionArg());
			}
			else if (args.OptionId() == OPT_LC_FILTERING )
			{
				p.__complexity = COMPLEXITY_NONE;
			}
			else if (args.OptionId() ==  OPT_VERBOSE )
			{
				p.SetVerbose();
			}
			else if (args.OptionId() ==  OPT_SILENT )
			{
				p.SetSilent();
			}
			else if (args.OptionId() == OPT_BANK_NAME )
			{
				p.__bank_name = (string) args.OptionArg();
			}
			else if (args.OptionId() == OPT_NB_PASS )
			{
				p.__bank_nb_ind_pass = atoi(args.OptionArg());
			}
			else if (args.OptionId() == OPT_MAX_COUNT )
			{
				p.__black_list_max_count = atoi(args.OptionArg());
			}
			else if (args.OptionId() == OPT_MASK)
			{
				p.__msk_string = (string) args.OptionArg();
			}

			#ifdef DEBUGLOG
			else if (args.OptionId() ==  OPT_LOG_LEVEL)
			{
				p.__log_level = (string) args.OptionArg();
			}
			#endif
		}
        else 
        {
            cerr << "\nERROR - Invalid argument: " << args.OptionText() << "\n\n";
            cmd.Usage();
            exit(1);	
        }
    }
    return args;
}

/**
 * \brief called by _Init to finish the job
 */
void ParametersInitializerIndex::_Init2 ( const CSimpleOpt& args, const Command& )
{
	if ( args.FileCount()>=1 )
	{
		p.__bank_filename = (CompressedFileName) args.File(0);
	}
	p.__InitBankName();
}

/** 
 *  \brief Validate the parameters useful for glintindex
 *  \return true if all good
*/
bool ParametersInitializerIndex::_Validation()
{
	bool ok = true;
	if ( p.__argc == 1 )
	{
		ok = false;
	}
    if (!p.__bank_filename)
    {
		cerr << "\nERROR - Illegal bank file name (empty string)\n\n";
		ok = false;
	};
	return ok;
}
