/*
   $Id: ParametersMap.cpp 817 2017-04-06 11:41:21Z tfaraut $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/
#include "ParametersMap.h"
#include "Command.h"

CSimpleOpt ParametersInitializerMap::_Init(int argc, char* argv[], const Command& cmd)
{
	CSimpleOpt::SOption options[] = {
	    // ID       TEXT          TYPE
		//	Specific options of the glint executable
	    { OPT_CUTOFF_SLIDE,  (char*)"-c",    SO_REQ_SEP }, // "-c"
	    { OPT_CUTOFF_DYN,  (char*)"-X",      SO_REQ_SEP }, // "-X"
	    { OPT_CUTOFF_HSP,  (char*)"-s",      SO_REQ_SEP }, // "-s"
	    { OPT_LOWER_LEVEL,  (char*)"-w",     SO_REQ_SEP }, // "-w"
	    { OPT_DEEPEST_LEVEL, (char*) "-d",   SO_REQ_SEP }, // "-d"
	    { OPT_COMPLEXITY,    (char*) "-C",   SO_REQ_SEP }, // "-C"
	    { OPT_LC_FILTERING,  (char*) "--no-lc-filtering", SO_NONE }, // "--no-lc-filtering"
	    { OPT_OUTPUTFORMAT,  (char*)"-m",    SO_REQ_SEP }, // "-m"
	    { OPT_OUTPUTFORMAT,  (char*)"--output-format",SO_REQ_SEP }, // "--output-format"
	    { OPT_INPUTFORMAT,   (char*)"--input-format", SO_REQ_SEP }, // "--input-format"
	    { OPT_RESULTSFILENAME,  (char*)"-o", SO_REQ_SEP }, // "-o"
	    { OPT_RESULTSFILENAME,  (char*)"--output-file", SO_REQ_SEP }, // "--output-file"
	    { OPT_PIPE, (char*) "-",          SO_NONE    }, // "-"
	    { OPT_TOUPPER, (char*) "--to-upper", SO_NONE }, // "--to-upper"
	    { OPT_STRICT,  (char*)"--strict",   SO_NONE     }, //
	    { OPT_STEP,  (char*)"--step",       SO_REQ_SEP  }, //
	    { OPT_SILENT,  (char*)"--silent",   SO_NONE }, //
	    { OPT_VERBOSE,  (char*)"--verbose", SO_NONE }, //
	    { OPT_VERBOSE,  (char*)"-v",        SO_NONE }, //
	    { OPT_BEST, (char*)"--best-score", SO_NONE }, // --best-score
	    { OPT_BWALIKE_OUTPUT, (char*)"--bwa-like", SO_NONE }, // --bwa-like
	
	   	#ifdef _OPENMP
		// asking for a number of threads
		{ OPT_A,          (char*) "-a", SO_REQ_SEP }, // -a
		#endif
	
	     // Options for the alignments socring scheme 
	    { OPT_MATCH, (char*)"--match", SO_REQ_CMB   }, // 
		{ OPT_MISMATCH, (char*)"--mismatch", SO_REQ_CMB   }, //
		{ OPT_GAP_OPENING, (char*)"--gapop", SO_REQ_CMB   }, // 
		{ OPT_GAP_EXTENSION, (char*)"--gapext", SO_REQ_CMB   }, //
        { OPT_UNGAP_DROPOFF, (char*)"--ungapped-dropoff", SO_REQ_CMB   }, //
        { OPT_GAP_DROPOFF, (char*)"--gapped-dropoff", SO_REQ_CMB   }, //
	    { OPT_NOGAP,  (char*)"--no_gap",    SO_NONE }, //
		{ OPT_NOGAP,  (char*)"--no-gap",    SO_NONE }, //
		{ OPT_MAX_MIS, (char*)"--mmis",     SO_REQ_SEP }, //
	    { OPT_MAX_GAP, (char*)"--mgap",     SO_REQ_SEP }, //
	    { OPT_SIZE_MIN, (char*)"--lmin",    SO_REQ_SEP }, //
	    { OPT_IDENT_MIN, (char*)"--imin",    SO_REQ_SEP }, // min sequence identity
	    { OPT_SCORE_RATIO_MIN, (char*)"--srmin",  SO_REQ_SEP }, //
	    { OPT_LENGTH_RATIO_MIN, (char*)"--lrmin", SO_REQ_SEP }, //
	#ifdef DEBUGLOG
	    { OPT_LOG_LEVEL,  (char*)"-D",     SO_REQ_SEP }, // "-D"
	#endif
	
		// writing compressed data
		{ OPT_COMPRESS,   (char*) "--compress",     SO_REQ_SEP },
		
		// saving the index when using glint, index mode
		{ OPT_SAVE_INDEX, (char*) "--save-index", SO_NONE }, // --save-index
		
	    //	Options specific to the indexation process
	    { OPT_MAX_COUNT, (char*) "-n",     SO_REQ_SEP    }, // "-n"
	    { OPT_MASK,  (char*) "-M",     SO_REQ_SEP    },     // "-M"
	    { OPT_MASK,  (char*) "--mask", SO_REQ_SEP    },     // "---mask"
	    { OPT_BANK_NAME,  (char*)"-N",     SO_REQ_SEP   },  // "-w"
	    { OPT_NB_PASS,  (char*)"-p",     SO_REQ_SEP    },   // "-p"
	    
	    // To get some help
	    { OPT_HELP, (char*)"-?",     SO_NONE    }, // "-?"
	    { OPT_HELP, (char*)"-h",     SO_NONE    }, // "-?"
	    { OPT_HELP, (char*)"--help", SO_NONE    }, // "--help"
	    
	    SO_END_OF_OPTIONS                       // END
	};

	//p.__Help = help;
	p.__argc = argc;
	p.__high_throughput_mode = true; // The map command is high throughput

	// declare our options parser, pass in the arguments from main
    // as well as our array of valid options.
    CSimpleOpt args(argc, argv, options);
	while (args.Next()) {
		if (args.LastError() == SO_SUCCESS) 
		{
			if (args.OptionId() == OPT_HELP) 
			{
				cmd.Usage();
				exit(0);
			}
	        else if (args.OptionId() == OPT_CUTOFF_SLIDE )
			{
				p.__cutoff_slide = atoi(args.OptionArg());
			}
			else if (args.OptionId() == OPT_CUTOFF_DYN )
			{
				p.__cutoff_dyn = atoi(args.OptionArg());
			}
			else if (args.OptionId() == OPT_CUTOFF_HSP )
			{
				p.__cutoff_hsp = atoi(args.OptionArg());
			}
			else if (args.OptionId() == OPT_LOWER_LEVEL )
			{
				p.__lower_level = atoi(args.OptionArg());
				p.__deepest_level = p.__lower_level;
			}
			else if (args.OptionId() == OPT_COMPLEXITY )
			{
				p.__complexity = atoi(args.OptionArg());
			}
			else if (args.OptionId() == OPT_LC_FILTERING )
			{
				p.__complexity = COMPLEXITY_NONE;
			}
			else if (args.OptionId() == OPT_OUTPUTFORMAT)
			{
				string out = args.OptionArg();
				if ( ToLower(out) == "fa" || ToLower(out) == "fasta" )
				{
					p.__output_format = OUTPUT_FORMAT_FA;
				}
				else if ( ToLower(out) == "fastq" )
				{
					p.__output_format = OUTPUT_FORMAT_FASTQ;
				}
				else if ( ToLower(out) == "sam" )
				{
					p.__output_format = OUTPUT_FORMAT_SAM;
				}
				else if ( ToLower(out) == "bam" )
				{
					p.__output_format = OUTPUT_FORMAT_BAM;
				}
				else if ( ToLower(out) == "samext" )
				{
					p.__output_format = OUTPUT_FORMAT_SAMEXT;
				}
				else if ( ToLower(out) == "bamext" )
				{
					p.__output_format = OUTPUT_FORMAT_BAMEXT;
				}
				else if ( ToLower(out) == "bed" )
				{
					p.__output_format = OUTPUT_FORMAT_BED;
				}
				else if ( out == "0" )
				{
					p.__output_format = OUTPUT_FORMAT_M0;
				}
				else
				{
					int output_format = atoi(args.OptionArg());
					if (output_format==0)
					{
						p.__output_format = OUTPUT_FORMAT_ILLEGAL;
					}
					else
					{
						p.__output_format = output_format;
					}
				}
			}
			else if (args.OptionId() == OPT_INPUTFORMAT)
			{
				string in = args.OptionArg();
				if ( ToLower(in) == "fa" || ToLower(in) == "fasta")
				{
					p.__input_format = INPUT_FORMAT_FA;
				}
				else if ( ToLower(in) == "fastq" )
				{
					p.__input_format = INPUT_FORMAT_FASTQ;
				}
				else if ( ToLower(in) == "sam" || ToLower(in) == "samext")
				{
					p.__input_format = INPUT_FORMAT_SAM;
				}
				else if ( ToLower(in) == "bam" || ToLower(in) == "bamext" )
				{
					p.__input_format = INPUT_FORMAT_BAM;
				}
				else
				{
					string msg = "ERROR - ";
					msg += in;
					msg += " Invalid value for --input-format";
					throw(runtime_error(msg));
				}
			}          
			else if (args.OptionId() == OPT_RESULTSFILENAME )
			{
				string results_filename = (string) args.OptionArg();
				string bwd_filename;
				p.__ParseFileName(results_filename,bwd_filename);
				p.__results_filename.SetFileName(results_filename);
				p.__results_bwd_filename.SetFileName(bwd_filename);
			}
			else if (args.OptionId() ==  OPT_PIPE )
			{
				p.__pipe = true;
			}
			else if (args.OptionId() ==  OPT_TOUPPER )
			{
				p.__to_upper = true;
			}
			else if (args.OptionId() ==  OPT_MATCH )
	        {
	            p.__match = atoi(args.OptionArg());
	        }
			else if (args.OptionId() ==  OPT_MISMATCH )
			{	            	
				p.__mismatch = atoi(args.OptionArg());
			}
			else if (args.OptionId() ==  OPT_GAP_OPENING )
			{
				p.__gapop = atoi(args.OptionArg());
			}
			else if (args.OptionId() ==  OPT_GAP_EXTENSION )
			{
				p.__gapext = atoi(args.OptionArg());
			}
            else if (args.OptionId() ==  OPT_UNGAP_DROPOFF )
			{
				p.__ungapdropoff = atoi(args.OptionArg());
			}
            else if (args.OptionId() ==  OPT_GAP_DROPOFF )
			{
				p.__gapdropoff = atoi(args.OptionArg());
			}	
			else if (args.OptionId() ==  OPT_STRICT )
			{
				p.__strict = true;
			}
			else if (args.OptionId() ==  OPT_NOGAP )
			{
				p.__dyn = false;
			}
			else if (args.OptionId() ==   OPT_MAX_MIS )
			{
				p.__max_mismatch = atoi(args.OptionArg());
			}
			else if (args.OptionId() ==   OPT_MAX_GAP )
			{
				p.__max_gap = atoi(args.OptionArg());
			}
			else if (args.OptionId() ==   OPT_SIZE_MIN )
			{
				p.__size_min = atoi(args.OptionArg());
			}
			else if (args.OptionId() ==   OPT_IDENT_MIN )
			{
				p.__ident_min = atoi(args.OptionArg());
			}
			else if (args.OptionId() ==   OPT_SCORE_RATIO_MIN )
			{
				p.__min_ratio_of_max_score = atof(args.OptionArg());
			}
			else if (args.OptionId() ==   OPT_LENGTH_RATIO_MIN )
			{
				p.__min_ratio_of_max_length = atof(args.OptionArg());
			}
			else if (args.OptionId() ==  OPT_STEP )
			{
				p.__step = atoi(args.OptionArg());
			}	           
			else if (args.OptionId() ==  OPT_VERBOSE )
			{
				p.SetVerbose();
			}
			else if (args.OptionId() ==  OPT_SILENT )
			{
				p.SetSilent();
			}
			else if (args.OptionId() ==  OPT_BEST )
			{
				p.__best_score = true;
			}
			else if (args.OptionId() ==  OPT_BWALIKE_OUTPUT ) // If bwalike output is asked best_score is true (only one line per query)
			{	
				p.__bwalike = true;
			}
			else if (args.OptionId() == OPT_BANK_NAME )
			{
				p.__bank_name = (string) args.OptionArg();
			}
			else if (args.OptionId() == OPT_NB_PASS )
			{
				p.__bank_nb_ind_pass = atoi(args.OptionArg());
			}
			else if (args.OptionId() == OPT_MAX_COUNT )
			{
				p.__black_list_max_count = atoi(args.OptionArg());
			}
			else if (args.OptionId() == OPT_MASK)
			{
				p.__msk_string = (string) args.OptionArg();
			}
			else if ( args.OptionId() == OPT_COMPRESS )
			{
				p.__results_filename.SetCompression((string) args.OptionArg());
				p.__results_bwd_filename.SetCompression((string) args.OptionArg());
			}
			else if ( args.OptionId() == OPT_SAVE_INDEX )
			{
				p.__save_index_mode = true;
			}
			#ifdef _OPENMP
			else if ( args.OptionId() == OPT_A )
			{
				string arg = args.OptionArg();
				if ( arg == "auto" )
				{
					p.__threads_nb = 0;
				}
				else
				{
					p.__threads_nb = (uint8_t) atoi(args.OptionArg());
				}
			}
			#endif
				
			#ifdef DEBUGLOG
			else if (args.OptionId() ==  OPT_LOG_LEVEL)
			{
				p.__log_level = (string) args.OptionArg();
			}
			#endif
		}
        else 
        {
            fprintf(stderr,"Invalid argument: %s\n", args.OptionText());
            fprintf(stderr,"Please use option --help to see the usage\n");
            exit(1);	
        }
    }
    return args;
}

/**
 * \brief called by Init to finish the job
 */
void ParametersInitializerMap::_Init2( const CSimpleOpt & args, const Command& cmd)
{
    if (p.__pipe)
    {
	    if (args.FileCount()!=1)
		{
			cerr << "\nERROR - Exactly one filename is required when the request is read from stdin\n\n";
			cmd.Usage();
			exit(1);
		}
		else
		{
			p.__bank_filename = (CompressedFileName) args.File(0);
		}
		p.__query_filename = (CompressedFileName) "-";
	}
	else
	{
		if ( args.FileCount() >= 1 )
		{
			p.__bank_filename  = (CompressedFileName) args.File(0);
		}
		if ( args.FileCount() >= 2 )
		{
			string query_filename = args.File(1);
			string bwd_filename = "";
			p.__ParseFileName(query_filename,bwd_filename);
			p.__query_filename = (CompressedFileName) query_filename;
			p.__bwd_filename   = (CompressedFileName) bwd_filename;
		}
		if ( args.FileCount() >= 3 )
		{
			p.__bwd_filename = (CompressedFileName) args.File(2);
		}
		if (args.FileCount() > 3) 
		{
			cerr << "\nERROR - Too many input files specified\n\n";
			cmd.Usage();
			exit(1);
		}
	}
     
     // If the query file is compressed and no --compressed switch was given,
     // we'll compress the output with the same algorithm
     // try to guess this algorithm from the extension
     if ( !p.__pipe )
     {
		 if ( p.__results_filename.GetCompression() == NOCOMPRESSION )
		 {
			 p.__results_filename.SetCompression     ( p.__query_filename.GetCompression() );
			 p.__results_bwd_filename.SetCompression ( p.__query_filename.GetCompression() );
		 }
	 }
	
	// Force the pass nb to 1 if indexing in memory
	p.__index_mode = !(p.__FindIndex());
	if ( p.__index_mode && !p.__save_index_mode )
	{
		p.__bank_nb_ind_pass = 1;
	}
	
	p.__InitBankName();
	
	if ( p.__input_format == INPUT_FORMAT_UNKNOWN )
	{
		p.__input_format = DEFAULT_GLINT_INPUT_FORMAT;
	}
	if ( p.__output_format == OUTPUT_FORMAT_UNKNOWN )
	{
		p.__output_format = DEFAULT_GLINT_OUTPUT_FORMAT;
	}		
}

/** 
 *  \brief Validate the parameters useful for glint
 *  \return true if all good
 *  \private
*/
bool ParametersInitializerMap::_Validation()
{
	bool ok = true;
	if ( p.__argc == 1 )
	{
		ok = false;
	}
    if (!p.__bank_filename)
    {
		cerr << "\nERROR - Illegal bank file name\n\n";
		ok = false;
	}
    if (!p.__query_filename)
    {
		cerr << "\nERROR - Illegal query file name\n\n";
		ok = false;
	}
	if ( p.__bwd_filename)
	{
		cerr << "ERROR - Please use the mappe command of glint for pair end.\n";
		ok = false;
	}
	if ( p.__cutoff_hsp < p.__cutoff_dyn )
	{
		//Modified by Thomas 18/07/2011
		//cerr << "ERROR - s switch (" << p.__cutoff_hsp << ") < X switch (" << p.__cutoff_dyn << ")\n\n";	
		//ok = false;
		cerr<< "Warning: s switch (" << p.__cutoff_hsp << ") < X switch (" << p.__cutoff_dyn << "), automatically setting X to "<< p.__cutoff_hsp <<"\n\n ";
		p.__cutoff_dyn = p.__cutoff_hsp;
	}
	if ( p.__deepest_level < p.__lower_level ) 
	{
		cerr << "\nERROR - d switch (" << (int) p.__deepest_level << ") < w switch (" << (int) p.__lower_level << ")\n\n";	
		ok = false;
	}	
	if ( !p.__deepest_level || !p.__lower_level )
	{
		cerr << "\nERROR - d or w switch = 0\n\n";
		ok = false;
	}	
	
	if ( p.IsBwaLikeOutput() && p.IsBestScore() ) // Both cannot be set simultaneously
	{ 
		cerr << "\nERROR - both options best-score and bwa-like cannot be simultaneously true, please choose only one\n";
		ok = false;
	}
	
	if ( p.IsBwaLikeOutput() ) // We want full output to mimic the bwa output format, we need therefore SAM/BAM-like output
	{ 
		if ( p.GetOutputFormat() !=  OUTPUT_FORMAT_SAM && 
		     p.GetOutputFormat() !=  OUTPUT_FORMAT_BAM &&
		     p.GetOutputFormat() !=  OUTPUT_FORMAT_SAMEXT && 
		     p.GetOutputFormat() !=  OUTPUT_FORMAT_BAMEXT
		    )
		{
			cerr <<  "ERROR - If full output is asked (for the bwa-like output format) the output format must be of type SAM/BAM (ext) \n";
			ok = false;
		}	
	}
	
	return ok;
}
